#ifndef Data_H
#define Data_H

#include <Array.h>
#include <Punkt.h>
#include <Bild.h>
#include <Model2D.h>

using namespace Framework;

class VertexData
{
private:
	Vec2< float > vertex;
	Punkt textur;
	bool selected;
	bool sichtbar;
	int ref;

public:
	// Konstruktor
	VertexData( Vec2< float > v, Punkt t );
	// nicht constant
	void nachLinks( float num );
	void nachOben( float num );
	void saveTextur( Punkt tPos );
	void select( Punkt p1, Punkt p2 );
	void deSelect();
	void setAuswahl( bool ausw );
	void setSichtbar( bool s );
	// constant
	bool istSichtbar() const;
	bool istAusgewhlt() const;
	Vec2< float > getPos() const;
	Punkt getTPos() const;
	// Reference Counting
	VertexData *getThis();
	VertexData *release();
};

class PolygonData
{
private:
	RCArray< VertexData > *vd;
	bool sichtbar;
	int ref;

public:
	// Konstruktor
	PolygonData( Polygon2D &pg );
	// Destruktor
	~PolygonData();
	// nicht constant
	void addVertex( Vec2< float >v, Punkt t );
	void removeVertex( int num );
	void nachLinks( float num );
	void nachOben( float num );
	void saveTextur( Punkt tPos );
	void select( Punkt p1, Punkt p2 );
	void deSelect();
	void setSichtbar( bool s );
	// constant
	bool istSichtbar() const;
	VertexData *zVertex( int num ) const;
	int getVertexAnzahl() const;
	void getM2( Polygon2D &pd, bool textur ) const;
	// Reference Counting
	PolygonData *getThis();
	PolygonData *release();
};

class Data
{
private:
	RCArray< PolygonData > *pd;
	Bild *textur;
	Punkt tPos;
	bool rTextur;
	int sp;
	int ref;

public:
	// Konstruktor
	Data( Model2DData *mdl );
	// Destruktor
	~Data();
	// nicht constant
	void addPolygon();
	void removePolygon( int num );
	void selectPolygon( int num );
	void nachLinks( float num );
	void nachOben( float num );
	void tNachLinks( int num );
	void tNachOben( int num );
	void setRTextur( bool rt );
	void saveTextur();
	void setTextur( Bild *t );
	void select( Punkt p1, Punkt p2 );
	void deSelect();
	// constant
	PolygonData *zPolygon( int num ) const;
	int getPolygonAnzahl() const;
	int getSelectedPolygon() const;
	Model2DData *getM2() const;
	Bild *zTextur() const;
	Punkt getTPos() const;
	// Reference Counting
	Data *getThis();
	Data *release();
};

#endif