#include "Editor2D.h"
#include <Rahmen.h>
#include <Text.h>
#include <Globals.h>
#include <TastaturEreignis.h>
#include <MausEreignis.h>

// Inhalt der Editor2D Klasse aus Editor2D.h
// Konstruktor
Editor2D::Editor2D( Schrift *zSchrift )
{
	pos.x = 10;
	pos.y = 10;
	offs.x = -360;
	offs.y = -360;
	mausPos.x = 0;
	mausPos.y = 0;
	data = 0;
	ram = new LRahmen();
	ram->setFarbe( 0xFFFFFFFF );
	ram->setRamenBreite( 1 );
	ram->setSize( 720, 720 );
	select = new LRahmen();
	select->setFarbe( 0xFF5050FF );
	select->setRamenBreite( 1 );
	schrift = zSchrift->getThis();
	addV = Vertex( 0, 0 );
	mausIn = 0;
	gre = 1;
	ref = 1;
}

// Destruktor
Editor2D::~Editor2D()
{
	ram->release();
	select->release();
	schrift->release();
	if( data )
		data->release();
}

// nicht constant
void Editor2D::setDataZ( Data *d )
{
	if( data )
		data->release();
	data = d;
}

void Editor2D::doMausEreignis( MausEreignis &me )
{
	me.mx -= pos.x;
	me.my -= pos.y;
	if( me.mx < 0 || me.my < 0 || me.mx > ram->getBreite() || me.my > ram->getHeight() )
	{
		me.mx += pos.x;
		me.my += pos.y;
		mausIn = 0;
		return;
	}
	mausIn = 1;
	rend = 1;
	addV = Vertex( (float)me.mx, (float)me.my ) / gre + offs;
	if( me.id == ME_UScroll )
		gre += 0.01f;
	if( me.id == ME_DScroll )
		gre -= 0.01f;
	if( me.id == ME_PLinks )
	{
		select->setPosition( me.mx, me.my );
		select->setSize( 0, 0 );
	}
	if( me.id == ME_PRechts || me.id == ME_PMitte )
	{
		mausPos.x = me.mx;
		mausPos.y = me.my;
	}
	if( me.id == ME_Bewegung )
	{
		if( getMausStand( M_Links ) )
		{
			select->setSize( me.mx - select->getX(), me.my - select->getY() );
		}
		if( getMausStand( M_Rechts ) )
		{
			data->nachLinks( ( mausPos.x - me.mx ) / gre );
			data->nachOben( ( mausPos.y - me.my ) / gre );
			mausPos.x = me.mx;
			mausPos.y = me.my;
		}
		if( getMausStand( M_Mitte ) )
		{
			data->tNachLinks( (int)( ( mausPos.x - me.mx ) / gre ) );
			data->tNachOben( (int)( ( mausPos.y - me.my ) / gre ) );
			mausPos.x = me.mx;
			mausPos.y = me.my;
		}
	}
	if( me.id == ME_RLinks )
	{
		if( !getTastenStand( T_Shift ) )
			data->deSelect();
		data->select( ( Vec2<float> )select->getPosition() / gre + offs, ( Vec2<float> )( select->getPosition() + select->getSize() ) / gre + offs );
		select->setSize( 0, 0 );
		select->setPosition( 0, 0 );
	}
	if( me.id == ME_RRechts )
	{
		mausPos.x = 0;
		mausPos.y = 0;
	}
	me.verarbeitet = 1;
	me.mx += pos.x;
	me.my += pos.y;
}

void Editor2D::doTastaturEreignis( TastaturEreignis &te )
{
	if( te.id == TE_Release && te.taste == T_Enter && mausIn )
	{
		PolygonData *pd = data->zPolygon( data->getSelectedPolygon() );
		if( pd )
		{
			pd->addVertex( addV, Punkt( 0, 0 ) );
			rend = 1;
		}
	}
	if( te.id == TE_Release && te.taste == T_Einfg )
	{
		data->addPolygon();
		rend = 1;
	}
}

bool Editor2D::tick( double zeit )
{
	if( mausIn )
	{
		if( getTastenStand( T_Links ) )
		{
			offs.x--;
			rend = 1;
		}
		if( getTastenStand( T_Rechts ) )
		{
			offs.x++;
			rend = 1;
		}
		if( getTastenStand( T_Oben ) )
		{
			offs.y--;
			rend = 1;
		}
		if( getTastenStand( T_Unten ) )
		{
			offs.y++;
			rend = 1;
		}
		if( getTastenStand( 'w' ) )
		{
			data->tNachOben( 1 );
			rend = 1;
		}
		if( getTastenStand( 'a' ) )
		{
			data->tNachLinks( 1 );
			rend = 1;
		}
		if( getTastenStand( 's' ) )
		{
			data->tNachOben( -1 );
			rend = 1;
		}
		if( getTastenStand( 'd' ) )
		{
			data->tNachLinks( -1 );
			rend = 1;
		}
	}
	bool ret = rend;
	rend = 0;
	return ret;
}

void Editor2D::render( Bild &zRObj )
{
	if( !zRObj.setDrawOptions( pos, ram->getSize() ) )
		return;
	ram->render( zRObj );
	if( !zRObj.setDrawOptions( 1, 1, ram->getBreite() - 2, ram->getHeight() - 2 ) )
	{
		zRObj.releaseDrawOptions();
		return;
	}
	if( data->zTextur() )
	{
		Punkt tPos = data->getTPos();
		zRObj.alphaBildSkall( (int)( ( tPos.x - offs.x ) * gre ), (int)( ( tPos.y - offs.y ) * gre ),
							  (int)( data->zTextur()->getBreite() * gre ), (int)( data->zTextur()->getHeight() * gre ), *data->zTextur() );
	}
	// Raster mahlen
	int anz = (int)( 720 / ( 50 * gre ) );
	int xStart = ( 50 - abs( offs.x ) % 50 );
	if( offs.x < 0 )
		xStart = -offs.x % 50;
	if( offs.x == 0 )
		xStart = 0;
	int yStart = ( 50 - abs( offs.y ) % 50 );
	if( offs.y < 0 )
		yStart = -offs.y % 50;
	if( offs.y == 0 )
		yStart = 0;
	for( float x = xStart * gre, y = yStart * gre; !( x > 720 && y > 720 ); x += 50 * gre, y += 50 * gre )
	{
		zRObj.drawLinieH( 0, (int)y, 720, 0xFF505050 );
		zRObj.drawLinieV( (int)x, 0, 720, 0xFF505050 );
	}
	Text xPos = "";
	xPos = offs.x + (int)( xStart + 50 * ( anz / 2 ) );
	schrift->setDrawPosition( (int)( xStart * gre + 50 * gre * ( anz / 2 ) ), 0 );
	schrift->renderText( &xPos, zRObj, 0xFF505050 );
	xPos = offs.y + (int)( yStart + 50 * ( anz / 2 ) );
	schrift->setDrawPosition( 0, (int)( yStart * gre + 50 * gre * ( anz / 2 ) ) );
	schrift->renderText( &xPos, zRObj, 0xFF505050 );
	// Model mahlen
	int pAnz = data->getPolygonAnzahl();
	for( int i = 0; i < pAnz; i++ )
	{
		PolygonData *p = data->zPolygon( i );
		if( !p->istSichtbar() )
			continue;
		if( data->getSelectedPolygon() == i && mausIn )
		{
			int vAnz = p->getVertexAnzahl();
			VertexData tmp = VertexData( addV, Punkt() );
			VertexData *l = p->zVertex( vAnz - 1 );
			for( int j = -1; j < vAnz && vAnz > 0; j++ )
			{
				VertexData *v = j < 0 ? &tmp : p->zVertex( j );
				if( l && v )
				{
					if( l->istSichtbar() && v->istSichtbar() )
						zRObj.drawLinie( ( ( l->getPos() - offs ) * gre ), ( ( v->getPos() - offs ) * gre ), 0xFFA0A0A0 );
					else
						zRObj.drawLinie( ( ( l->getPos() - offs ) * gre ), ( ( v->getPos() - offs ) * gre ), 0xFF606060 );
				}
				if( !l->istSichtbar() )
				{
					l = v;
					continue;
				}
				if( j == 0 )
					zRObj.fillRegion( (int)( ( l->getPos().x - offs.x ) * gre ) - 5, (int)( ( l->getPos().y - offs.y ) * gre ) - 5, 10, 10, 0xFF50FF50 );
				else
				{
					if( !l->istAusgewhlt() )
						zRObj.fillRegion( (int)( ( l->getPos().x - offs.x ) * gre ) - 5, (int)( ( l->getPos().y - offs.y ) * gre ) - 5, 10, 10, 0xFF5050FF );
					else
						zRObj.fillRegion( (int)( ( l->getPos().x - offs.x ) * gre ) - 5, (int)( ( l->getPos().y - offs.y ) * gre ) - 5, 10, 10, 0xFFFF5050 );
				}
				l = v;
			}
		}
		else
		{
			int vAnz = p->getVertexAnzahl();
			VertexData *l = p->zVertex( vAnz - 1 );
			for( int j = 0; j < vAnz; j++ )
			{
				VertexData *v = p->zVertex( j );
				if( l && v )
				{
					if( l->istSichtbar() && v->istSichtbar() )
						zRObj.drawLinie( ( ( l->getPos() - offs ) * gre ), ( ( v->getPos() - offs ) * gre ), 0xFFA0A0A0 );
					else
						zRObj.drawLinie( ( ( l->getPos() - offs ) * gre ), ( ( v->getPos() - offs ) * gre ), 0xFF606060 );
				}
				if( !l->istSichtbar() )
				{
					l = v;
					continue;
				}
				if( !l->istAusgewhlt() )
					zRObj.fillRegion( (int)( ( l->getPos().x - offs.x ) * gre ) - 5, (int)( ( l->getPos().y - offs.y ) * gre ) - 5, 10, 10, 0xFF5050FF );
				else
					zRObj.fillRegion( (int)( ( l->getPos().x - offs.x ) * gre ) - 5, (int)( ( l->getPos().y - offs.y ) * gre ) - 5, 10, 10, 0xFFFF5050 );
				l = v;
			}
		}
	}
	select->render( zRObj );
	zRObj.releaseDrawOptions();
	zRObj.releaseDrawOptions();
}

// Reference Counting
Editor2D *Editor2D::getThis()
{
	ref++;
	return this;
}

Editor2D *Editor2D::release()
{
	ref--;
	if( !ref )
		delete this;
	return 0;
}